/* JOEUNPAK - Unpack a JOESND type archive */

#include <stdio.h>

#include "joe.h"

unsigned long output_chunk(char* input_name, char* output_name, struct joe_index idx)
{
   FILE* input_file;
   FILE* output_file;
   unsigned long real_offset;
   int tmp;
   unsigned long i;

   input_file = fopen(input_name, "rb");
   output_file = fopen(output_name, "wb");

   if(!input_file || !output_file)
   {
      printf("Error exporting chunk.\n");
      return 0;
   }

   real_offset = idx.offset_factor * OFFSET_MULTIPLIER;

   if ( (tmp = fseek(input_file, real_offset, SEEK_SET)) )
   {
      printf("Error: Couldn't seek to specified offset %ld\n", real_offset);
      return 0;
   }

   for (i = 0; i < idx.num_bytes; ++i)
   {
      tmp = fgetc(input_file);
      if (tmp == EOF)
      {
         printf("Error: unexpected EOF when exporting chunk.\n");
         return 0;
      }
      else
      {
         fputc(tmp, output_file);
      }
   }
   fclose(input_file);
   fclose(output_file);

   return i;  /* Returns byte count */
}


int main(int argc, char* argv[])
{
   FILE* input_file = NULL;
   FILE* list_file = NULL;
   char output_name[FILENAME_MAX];
   struct joe_index curr_idx;
   unsigned long tmp;
   int count;

   tmp = count = 0;

   if (argc < 2 || argc > 3)
   {
      printf("Usage: JOEUNPAK <file name> <list name>\n");
      printf("List file is optional and will export a list suitable for repacking.\n");
      return -1;
   }

   input_file = fopen(argv[1], "rb");

   if(!input_file)
   {
      printf("Error: Couldn't access %s\n", argv[1]);
      return -1;
   }

   if (argc == 3)
   {
      list_file = fopen(argv[2], "w");
      if(!list_file)
      {
         printf("Error: Couldn't access %s\n", argv[2]);
         return -1;
      }
   }

   if ( (tmp = fseek(input_file, -4096, SEEK_END)) )
   {
      printf("Error: Got return value %ld from fseek.\n", tmp);
      return -1;
   }

   do
   {
      fread(&curr_idx.offset_factor, 4, 1, input_file);
      fread(&curr_idx.num_bytes, 4, 1, input_file);
      fread(&curr_idx.extra, 4, 1, input_file);
      if(curr_idx.num_bytes != 0)
      {
         sprintf(output_name, "CHUNK%.3d", count);
         tmp = output_chunk(argv[1], output_name, curr_idx);
         /* Debug: */
         /* printf("Output %lu bytes, was told to output %lu bytes\n", tmp, curr_idx.num_bytes); */
         ++count;
         if(list_file)
         {
            fprintf(list_file, "%s %lu\n", output_name, curr_idx.extra);
         }
      }
   } while (curr_idx.num_bytes != 0 && count < MAX_ENTRIES);

   fclose(input_file);
   if(list_file)
      fclose(list_file);

   printf("%d chunks.\n", count);
   return 0;
}
